import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  scaleLinear,
  extent,
  max,
  format
} from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { Dropdown } from './Dropdown';

const width = window.innerWidth;
const menuHeight = 75;
const height = window.innerHeight - menuHeight;
const margin = {
  top: 20,
  right: 30,
  bottom: 65,
  left: 90,
};
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;

const attributes = [
   {value: 'sepal_length', label:'Sepal length'}, 
   {value: 'sepal_width', label:'Sepal width'}, 
   {value:'petal_length', label:'Petal length'}, 
   {value:'petal_width', label:'Petal width'},
   {value:'species', label:'Species'}
];

const xAxisLabelOffset = 50;
const yAxisLabelOffset = 40;

const App = () => {
  const data = useData();
  
  let initialXAttribute = 'petal_length';
  const [xAttribute, setXAttribute] = useState(
    initialXAttribute
  );
  const xValue = (d) => d[xAttribute];
  const xAxisLabel = 'Petal length';
  
  let initialYAttribute = 'sepal_width';
  const [yAttribute, setYAttribute] = useState(
    initialYAttribute
  );
  
  const yValue = (d) => d[yAttribute];
  const yAxisLabel = 'Sepal width';

  if (!data) {
    return <pre>Loading...</pre>;
  }
  console.log(data.columns);
  
  const siFormat = format('.2s');
  const xAxisTickFormat = n => siFormat(n).replace('G', 'B')

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
  	.nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  return (
    <>
    <label for="x-select">X:</label>
    <Dropdown
    	options={attributes}
      id="x-select"
      selectedValue={xAttribute}
      onSelectedValueChange={setXAttribute}  
    />
    <label for="y-select">Y:</label>
    <Dropdown
    	options={attributes}
      id="y-select"
      selectedValue={yAttribute}
      onSelectedValueChange={setYAttribute}  
    />
    <svg width={width} height={height}>
      <g
        className="tick"
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <AxisBottom
          tickFormat={xAxisTickFormat}
          xScale={xScale}
          innerHeight={innerHeight}
          tickOffset={5}
        />
        <AxisLeft 
          yScale={yScale} 
          innerWidth={innerWidth} 
          tickOffset={-5}
        />
        <text 
          className="axis-label"
          x={innerWidth / 2} 
          textAnchor="middle"
        	y={innerHeight + xAxisLabelOffset}  
        >
          {xAxisLabel}
        </text>
        <text
          transform={`rotate(-90) translate(${-innerHeight / 2}, ${-yAxisLabelOffset})`}
          className= "axis-label"
          text-anchor="middle" 
        >
          {yAxisLabel}
        </text>
        <Marks
          data={data}
          xScale={xScale}
          yScale={yScale}
          xValue={xValue}
          yValue={yValue}
          tooltipFormat={xAxisTickFormat}
          circleRadius={7}
        />
      </g>
    </svg>
      </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
